'use strict';

exports.__esModule = true;

var _instruction = require('../instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _nodeBuilder = require('../node-builder');

var _esotope = require('../tools/esotope');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Transform:
// const foo = window.postMessage; foo = window.postMessage; { _postMessage: window.postMessage }; return window.postMessage;
// -->
// const foo = _get$PostMessage(window.postMessage); foo = _get$PostMessage(window.postMessage); { _postMessage: _get$PostMessage(window.postMessage) }; return _get$PostMessage(window.postMessage);

exports.default = {
    nodeReplacementRequireTransform: false,

    nodeTypes: [_esotope.Syntax.MemberExpression],

    condition: function condition(node, parent) {
        // Skip: window.postMessage.field
        if (parent.type === _esotope.Syntax.MemberExpression && (parent.property === node || parent.object === node)) return false;

        // Skip: window.postMessage()
        if (parent.type === _esotope.Syntax.CallExpression && parent.callee === node) return false;

        // Skip: window.postMessage = 1, window["postMessage"] = 1
        if (parent.type === _esotope.Syntax.AssignmentExpression && parent.left === node) return false;

        // Skip already transformed: __get$PostMessage(window.postMessage), __get$PostMessage(window["postMessage"])
        if (parent.type === _esotope.Syntax.CallExpression && parent.callee.name === _instruction2.default.getPostMessage) return false;

        // window.postMessage
        if (node.property.type === _esotope.Syntax.Identifier && node.property.name === 'postMessage') return true;

        // window['postMessage']
        if (node.property.type === _esotope.Syntax.Literal && node.property.value === 'postMessage') return true;

        return false;
    },

    run: _nodeBuilder.createGetPostMessageMethCall
}; // -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

module.exports = exports['default'];