'use strict';

exports.__esModule = true;
exports.default = createShadowStylesheet;

var _css = require('css');

var _css2 = _interopRequireDefault(_css);

var _className = require('./class-name');

var _className2 = _interopRequireDefault(_className);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ID_OR_CLASS_RE = /#[a-zA-Z0-9_-]+|\.-?[a-zA-Z0-9_][a-zA-Z0-9_-]*/g;
var ADD_POSTFIX_REPLACEMENT = '$&' + _className2.default.postfix;

function transformSelector(selector) {
    return selector.replace(ID_OR_CLASS_RE, ADD_POSTFIX_REPLACEMENT);
}

function addUIClassPostfix(rules) {
    var ruleStack = rules.slice();

    while (ruleStack.length) {
        var rule = ruleStack.pop();

        if (rule.type === 'rule' && rule.selectors) rule.selectors = rule.selectors.map(transformSelector);

        if (rule.rules) ruleStack = ruleStack.concat(rule.rules);
    }
}

function createShadowStylesheet(cssCode) {
    var ast = _css2.default.parse(cssCode, { silent: true });

    addUIClassPostfix(ast.stylesheet.rules);

    return _css2.default.stringify(ast, {
        indent: 0,
        compress: false
    });
}
module.exports = exports['default'];